//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Viewer
//
//  MODULE:     perfdata.cpp
//
//  PURPOSE:    ,      
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include "stddef.h"
#include "perfdata.h"

///////////////////////////////////////////////////////////
//
// -
//
///////////////////////////////////////////////////////////

CPerformanceData::CPerformanceData() : 
						_called(false), 
						_data(NULL), 
						_size(0), 
						_actual_size(0)
{
	_hkpd = HKEY_PERFORMANCE_DATA;
}

CPerformanceData::~CPerformanceData()
{
	if (_called)
	{               
		RegCloseKey(_hkpd);
	}

	if (_data)
	{
		free(_data);
	}
}

///////////////////////////////////////////////////////////
//
// 
//
///////////////////////////////////////////////////////////

//  
//  FUNCTION:     int Connect(const TCHAR*)
//
//  PARAMETERS:   [IN] s -       NULL
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:           
//                  NULL   ,  
//                 .
//
//  COMMENTS:     
// 
int CPerformanceData::Connect(const TCHAR* s)
{
	if (_called)
	{          
		_called = false;
		RegCloseKey(_hkpd);
	}

	if ((s == NULL) || (s[0] == 0))
	{
		_hkpd = HKEY_PERFORMANCE_DATA;
		return NO_ERROR;
	}

	LONG res = RegConnectRegistry(s, HKEY_PERFORMANCE_DATA, &_hkpd);

	if (res != NO_ERROR)
	{
		_hkpd = HKEY_PERFORMANCE_DATA;
		return res;
	}

	_called = true;
	return res;
}

//  
//  FUNCTION:     int GoSmth(const TCHAR*)
//
//  PARAMETERS:   [IN] s - .   -   .
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          
//
//  COMMENTS:     
// 
int CPerformanceData::GoGlobal()
{
	return Go(TEXT("Global"));
}

int CPerformanceData::GoCostly()
{
	return Go(TEXT("Costly"));
}

int CPerformanceData::GoUser(const TCHAR* s)
{
	return Go(s);
}

//  
//  FUNCTION:     int ActualSizeInBytes() const
// 
//  PARAMETERS:   .
//
//  RETURN VALUE:    .
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         .    
//                   0.   0,    
//                 .
//
//  COMMENTS:     
// 
int CPerformanceData::ActualSizeInBytes() const
{
	return _actual_size;
}

//  
//  FUNCTION:     const void* Get() const
//
//  PARAMETERS:   
//
//  RETURN VALUE:   .
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         NULL,   .
//
//  COMMENTS:     
// 
const void* CPerformanceData::Get() const
{
	if (_actual_size == 0) return NULL;
	return _data;
}

///////////////////////////////////////////////////////////////////////
//
//  
//
///////////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int Go(const TCHAR*)
//
//  PARAMETERS:   [IN] s - .   -   .
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         . 
//
//  COMMENTS:       ""   .
// 
int CPerformanceData::Go(const TCHAR* s)
{
	if (!_called)
	{
		_called = true;
	}

	BYTE* tmp  = (BYTE*)_data;
	DWORD size = _size;
	DWORD type;
	LONG  res = 0;

	_actual_size = 0;

	for (;;)
	{
		res = RegQueryValueEx(_hkpd, s, 0, &type, tmp, &size);
	
		if (res == ERROR_SUCCESS)
		{
			break;
		}

		if (res != ERROR_MORE_DATA)
		{
			return res;
		}

		size += 4096;

		tmp = (BYTE*)realloc(_data, size);
		
		if (tmp == NULL)
		{
			return ERROR_OUTOFMEMORY;
		}

		_data = tmp;
		_size = size;
	}

	_actual_size = size;
	
	return 0;
}

////////////////////////////////////////////////////////////////////////

