#include "stddef.h"
#include "perfdata.h"
#include "perfobjects.h"
#include "getperf.h"

namespace get_perf
{
	//         
	int get_object(CPerformanceData& pd, int index, perf_object_type* obj);

	//   
	int get_counter(const perf_object_type& obj, int index, const PERF_COUNTER_DEFINITION** pc);

	//   
	int get_instance(const perf_object_type& obj, const WCHAR* name, perf_instance_definition* pi);

	//        
	// .    
	// .
	CPerformanceData data;

	//   .
	const int processor_index = 238;
	const int processor_processor_time_index = 6;

	const int process_index = 230;
	const int process_id_process_index = 784;
}

//////////////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////////////

//
//    "Processor Time"  "_Total"  "Processor"
int get_perf::processor_time(LARGE_INTEGER* value)
{
	perf_object_type object(0);

	//   "Processor"
	int res = get_object(data, processor_index, &object);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	const PERF_COUNTER_DEFINITION* proc_time = 0;

	//       "Processor Time"
	res = get_counter(object, processor_processor_time_index, &proc_time);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	perf_instance_definition total(0);

	//     "_Total"
	res = get_instance(object, L"_Total", &total);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	//  
	memcpy(value, total.get_counter_block() + proc_time->CounterOffset, sizeof(LARGE_INTEGER));
	
	return 0;
}

//
//    "ID Process"      "Process"
int get_perf::processes_info(DWORD** ids, WCHAR*** name, int* count)
{
	perf_object_type object(0);

	//   "Process"
	int res = get_object(data, process_index, &object);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	const PERF_COUNTER_DEFINITION* id_process = 0;

	//       "IP process"
	res = get_counter(object, process_id_process_index, &id_process);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	//     ID
	DWORD* temp_id = (DWORD*)malloc(object._data->NumInstances * sizeof(DWORD));

	if (temp_id == 0)
	{
		// ..
		return ERROR_NOT_ENOUGH_MEMORY;
	}

	const WCHAR** temp_name = (const WCHAR**)malloc(object._data->NumInstances * sizeof(WCHAR*));

	if (temp_id == 0)
	{
		//  ..
		free(temp_id);
		return ERROR_NOT_ENOUGH_MEMORY;
	}

	*ids   = temp_id;
	*name  = (WCHAR**)temp_name;
	*count = object._data->NumInstances;
	
	perf_object_type::instance_iterator ii = object.get_instance_iterator();
	perf_instance_definition instance(0);

	//  
	while (ii.is_any())
	{
		instance = ii.get_next();

		memcpy(temp_id, instance.get_counter_block() + id_process->CounterOffset, sizeof(DWORD));
		*temp_name = instance.name();

		temp_id++;
		temp_name++;
	}
	
	return 0;

}

//////////////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////////////

//
//   
int get_perf::get_object(CPerformanceData& pd, int index, perf_object_type* obj)
{
	TCHAR text_index[10];

	wsprintf(text_index, TEXT("%d"), index);

	//  
	int res = pd.GoUser(text_index);

	if (res != ERROR_SUCCESS)
	{
		// -  
		return res;
	}

	perf_data_block block(pd.Get());
	perf_data_block::iterator bi = block.get_iterator();
	perf_object_type object(0);

	//       
	while (bi.is_any())
	{
		object = bi.get_next();

		if (object._data->ObjectNameTitleIndex == index)
		{
			//  
			break;
		}
		else
		{
			//   
			object = NULL;
		}
	}

	if (object._data == 0) 
	{
		//  
		return ERROR_FILE_NOT_FOUND;
	}

	*obj = object;
	return 0;
}

//
//   
int get_perf::get_counter(const perf_object_type& obj, int index, const PERF_COUNTER_DEFINITION** pc)
{
	perf_object_type::counter_iterator ci = obj.get_counter_iterator();

	const PERF_COUNTER_DEFINITION* counter = 0;

	//   
	while (ci.is_any())
	{
		counter = ci.get_next();

		if (counter->CounterNameTitleIndex == index)
		{
			//  
			break;
		}
		else
		{
			//   
			counter = NULL;
		}
	}	
	
	if (counter == 0) 
	{
		//  
		return ERROR_FILE_NOT_FOUND;
	}

	*pc = counter;
	return 0;
}

//
//   
int get_perf::get_instance(const perf_object_type& obj, const WCHAR* name, perf_instance_definition* pi)
{
	perf_object_type::instance_iterator ii = obj.get_instance_iterator();
	perf_instance_definition instance(0);

	//  
	while (ii.is_any())
	{
		instance = ii.get_next();

		if (lstrcmpW(instance.name(), name) == 0)
		{
			//  
			break;
		}
		else
		{
			//   
			instance = NULL;
		}
	}

	if (instance._data == 0) 
	{
		//  
		return ERROR_FILE_NOT_FOUND;
	}

	*pi = instance;
	return 0;
}

////////////////////////////////////////////////////////////////
