//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Viewer
//
//  MODULE:     perfobjects.cpp
//
//  PURPOSE:     .   ,    
//                    
//               .    .
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//
//  CLASSES:
//      perf_data_block
//      perf_object_type
//      perf_instance_definition
//

#include "stddef.h"
#include "perfobjects.h"

////////////////////////////////////////////////////////////
//
//  perf_data_block
//
////////////////////////////////////////////////////////////

perf_data_block::perf_data_block(const void* d) : 
						_data((const PERF_DATA_BLOCK*)d)
{
}

const WCHAR* perf_data_block::name() const 
{
	return (const WCHAR*)((const char*)_data + _data->SystemNameOffset);
}
	
int perf_data_block::name_len() const 
{
	return _data->SystemNameLength / sizeof(WCHAR);
}

perf_data_block::iterator perf_data_block::get_iterator() const
{
	return iterator(_data);
}

////////////////////////////////////////////////////////////
//
//  perf_data_block::iterator
//
////////////////////////////////////////////////////////////

perf_data_block::iterator::iterator(const PERF_DATA_BLOCK* d): 
									_data(d), _offset(0), _num(0)
{
}
		
bool perf_data_block::iterator::is_any() const 
{
	return _num < _data->NumObjectTypes;
}

const PERF_OBJECT_TYPE* perf_data_block::iterator::get_first() 
{
	_offset = _data->HeaderLength;
	_num = 0;
	return get_next();
}

const PERF_OBJECT_TYPE* perf_data_block::iterator::get_next() 
{
	if (!is_any())    return NULL;
	if (_offset == 0) return get_first();
			
	const PERF_OBJECT_TYPE* temp = (const PERF_OBJECT_TYPE*)((const char*)_data + _offset);
	_offset += temp->TotalByteLength;
	_num++;

	return temp;
}

/////////////////////////////////////////////////////////////
//
//  perf_object_type
//
/////////////////////////////////////////////////////////////

perf_object_type::perf_object_type(const void* d) : 
						_data((const PERF_OBJECT_TYPE*)d) 
{
}

bool perf_object_type::is_instances() const
{
	return (_data->NumInstances != PERF_NO_INSTANCES);
}

perf_object_type::counter_iterator perf_object_type::get_counter_iterator() const
{
	return counter_iterator(_data);
}

perf_object_type::instance_iterator perf_object_type::get_instance_iterator() const
{
	return instance_iterator(_data);
}

/////////////////////////////////////////////////////////////
//
//  perf_object_type::counter_iterator
//
/////////////////////////////////////////////////////////////

perf_object_type::counter_iterator::counter_iterator(const PERF_OBJECT_TYPE* d) : 
											_data(d), _offset(0), _num(0)
{
}
		
bool perf_object_type::counter_iterator::is_any() const 
{
	return _num < _data->NumCounters;
}

const PERF_COUNTER_DEFINITION* perf_object_type::counter_iterator::get_first() 
{
	_offset = _data->HeaderLength;
	_num = 0;

	return get_next();
}

const PERF_COUNTER_DEFINITION* perf_object_type::counter_iterator::get_next() 
{
	if (!is_any()) return NULL;
	if (_offset == 0) return get_first();
			
	const PERF_COUNTER_DEFINITION* temp = (const PERF_COUNTER_DEFINITION*)((const char*)_data + _offset);
	_offset += temp->ByteLength;
	_num++;
			
	return temp;
}

/////////////////////////////////////////////////////////////
//
//  perf_object_type::instance_iterator
//
/////////////////////////////////////////////////////////////

perf_object_type::instance_iterator::instance_iterator(const PERF_OBJECT_TYPE* d) : 
											_data(d), _offset(0), _num(0)
{
}
		
bool perf_object_type::instance_iterator::is_any() const 
{
	return _num < _data->NumInstances;
}

const PERF_INSTANCE_DEFINITION* perf_object_type::instance_iterator::get_first() 
{
	_offset = _data->DefinitionLength;
	_num = 0;

	return get_next();
}

const PERF_INSTANCE_DEFINITION* perf_object_type::instance_iterator::get_next() 
{
	if (!is_any())    return NULL;
	if (_offset == 0) return get_first();

	const PERF_INSTANCE_DEFINITION* temp1 = (const PERF_INSTANCE_DEFINITION*)((const char*)_data + _offset);
	_offset += temp1->ByteLength;
	const PERF_COUNTER_BLOCK* temp2 = (const PERF_COUNTER_BLOCK*)((const char*)_data + _offset); 
	_offset += temp2->ByteLength;

	_num++;
			
	return temp1;
}

/////////////////////////////////////////////////////////////
//
//  perf_instance_definition
//
/////////////////////////////////////////////////////////////

perf_instance_definition::perf_instance_definition(const void* d) : 
								_data((const PERF_INSTANCE_DEFINITION*)d) 
{
}

const WCHAR* perf_instance_definition::name() const 
{
	if (   (_data->NameOffset == 0)
		|| (_data->NameLength == 0))
	{
		return NULL;
	}

	return (const WCHAR*)((const char*)_data + _data->NameOffset);
}
	
int perf_instance_definition::name_len() const 
{
	return _data->NameLength / sizeof(WCHAR);
}

const char* perf_instance_definition::get_counter_block() const
{
	return ((const char*)_data + _data->ByteLength);
}

////////////////////////////////////////////////////////////////////