//
//  Copyright (C) 2002, 
//
//  PROJECT:    
//
//  MODULE:     message.cpp
//
//  PURPOSE:     ,  ,   
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows 2000/XP/NT/9x
//
//  FUNCTIONS:
//       message()
//       question()
//       error()
//       format_error()
//

#include "stdwtl.h"
#include "resource.h"
#include "message.h"

namespace message
{
	int msgbox(HWND hwnd, const TCHAR* str, int flag = MB_OK);
}

//  
//  FUNCTION:     const TCHAR* get_title(void)
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       
//
//  COMMENTS:     
//
const TCHAR* message::get_title()
{
	static CString head;
	
	if (head.GetLength() == 0)
	{
		head.LoadString(IDS_TITLE);
	}

	return head;
}

//  
//  FUNCTION:     void message(HWND, int)
//
//  PARAMETERS:   [IN] hwnd -  
//                [IN] id   - ID 
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       
//
//  COMMENTS:     
//
void message::message(HWND hwnd, int id)
{
	CString s;
	s.LoadString(id);

	msgbox(hwnd, s);
}

//  
//  FUNCTION:     bool question(HWND, int)
//
//  PARAMETERS:   [IN] hwnd -  
//                [IN] id   - ID 
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       
//
//  COMMENTS:     
//
bool message::question(HWND hwnd, int id)
{
	CString s;
	s.LoadString(id);

	int res = msgbox(hwnd, s, MB_YESNOCANCEL);

	return (res == IDYES);
}

//  
//  FUNCTION:     bool question(HWND, int)
//
//  PARAMETERS:   [IN] hwnd -  
//                [IN] id   - ID 
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       
//
//  COMMENTS:     
//
int message::question_3(HWND hwnd, int id)
{
	CString s;
	s.LoadString(id);

	int res = msgbox(hwnd, s, MB_YESNOCANCEL);

	if (res == IDYES) return 1;
	if (res == IDNO)  return -1;
	
	return 0;
}

//  
//  FUNCTION:     void error(HWND, int)
//
//  PARAMETERS:   [IN] hwnd -  
//                [IN] id   - ID 
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       
//
//  COMMENTS:     
//
void message::error(HWND hwnd, int id)
{
	CString s;
	s.LoadString(id);

	msgbox(hwnd, s);
}

//  
//  FUNCTION:     void hresult_error(HWND, int)
//
//  PARAMETERS:   [IN] hwnd   -  
//                [IN] id     - ID 
//                [IN] err_id -   
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:          .
//
//  COMMENTS:     
//
void message::hresult_error(HWND hwnd, int id, int hr)
{
	CString s;
	s.LoadString(id);

	TCHAR text[1024];

	wsprintf(text, s, hr);

	msgbox(hwnd, text);
}

//  
//  FUNCTION:     void format_error(HWND, int)
//
//  PARAMETERS:   [IN] hwnd   -  
//                [IN] id     - ID 
//                [IN] err_id -   ( GetLastError)
//
//  RETURN VALUE: 
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:          
//
//  COMMENTS:     
//
void message::format_error(HWND hwnd, int id, int err_id)
{
	CString s;
	s.LoadString(id);

	TCHAR* description = NULL;

	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 
		NULL,  
		err_id, 
		0, 
		(LPTSTR) &description, 
		1, 
		NULL);

	TCHAR text[1024];

	wsprintf(text, s, description);

	LocalFree(description);

	msgbox(hwnd, text);
}

////////////////////////////////////////////////////////////////
//
//  
//
////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int  message::msgbox(HWND, const TCHAR*, int)
//
//  PARAMETERS:   [IN] hwnd -  
//                [IN] str  - 
//                [IN] flag - 
//
//  RETURN VALUE:  
//
//  REVISION:     19.07.2002   
//
//  PURPOSE:       MessageBox
//
//  COMMENTS:     
//
int  message::msgbox(HWND hwnd, const TCHAR* str, int flag)
{
	if (hwnd)
	{
		return MessageBox(hwnd, str, get_title(), flag);
	}
	else
	{
		return MessageBox(NULL, str, get_title(), flag | MB_TASKMODAL);
	}

}

////////////////////////////////////////////////////////////////////