#include "stdwtl.h"
#include "./perf/counterhelpdb.h"
#include "fieldslist.h"

namespace
{
//   3      2        2    2    2        1        1    1
//   1      8        4    2    0        6        2    0    8                0
//  +--------+--------+----+----+--------+--------+----+----+----------------+
//  |Display |Calculation  |Time|Counter |        |Ctr |Size|                |
//  |Flags   |Modifiers    |Base|SubType |Reserved|Type|Fld |   Reserved     |
//  +--------+--------+----+----+--------+--------+----+----+----------------+

	union CounterTypeMask
	{
		DWORD dw;

		struct
		{
			unsigned int reserved1 : 8;
			unsigned int size      : 2;
			unsigned int type      : 2;
			unsigned int reserved2 : 4;
			unsigned int subtype   : 4;
			unsigned int time      : 2;
			unsigned int calc      : 6;
			unsigned int display   : 4;
		};
	};

	// ,    
	struct column_info
	{
		int _head_id;
		int _cx;
		int _fmt;
	};

	const column_info columns[] = 
	{
		{IDS_H_FIELD, 100, LVCFMT_LEFT},
		{IDS_H_VALUE, 250, LVCFMT_LEFT},
	};
}

//////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////

CFieldsList::CFieldsList()
{
}

CFieldsList::~CFieldsList()
{
}

bool CFieldsList::Create(HWND parent)
{
	HWND hList;

	hList = CListViewCtrl::Create(
					parent, 
					NULL, 
					NULL, 
					WS_CHILD | WS_VISIBLE | WS_BORDER | LVS_NOSORTHEADER 
				  | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL,
					WS_EX_CLIENTEDGE);

	if (NULL == hList)
	{
		return false;
	}

	SetExtendedListViewStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);
	
	CString h;

	for (int i = 0; i < sizeof(columns)/sizeof(columns[0]); i++)
	{
		h.LoadString(columns[i]._head_id);
		InsertColumn(i, h, columns[i]. _fmt, columns[i]._cx, i);
	}

	return true;
}

void CFieldsList::Clear()
{
	DeleteAllItems();
}

//
//    
void CFieldsList::FillSystem(const PERF_DATA_BLOCK* system)
{
	Clear();

	AddTextW(TEXT("Signature"),      system->Signature, 4);
	AddHex(TEXT("LittleEndian"),     system->LittleEndian);
	AddHex(TEXT("Version"),          system->Version);
	AddHex(TEXT("Revision"),         system->Revision);
	AddHex(TEXT("TotalByteLength"),  system->TotalByteLength);
	AddHex(TEXT("HeaderLength"),     system->HeaderLength);
	AddHex(TEXT("NumObjectTypes"),   system->NumObjectTypes);
	AddHex(TEXT("DefaultObject"),    system->DefaultObject);
	AddTime(TEXT("SystemTime"),     &system->SystemTime);
	AddLarge(TEXT("PerfTime"),       system->PerfTime);
	AddLarge(TEXT("PerfFreq"),       system->PerfFreq);
	AddLarge(TEXT("PerfTime100nSec"),system->PerfTime100nSec);
	AddHex(TEXT("SystemNameLength"), system->SystemNameLength);
	AddHex(TEXT("SystemNameOffset"), system->SystemNameOffset);
	
	AddText(TEXT(""), TEXT(""));

	AddTextW(
		TEXT("SystemName"), 
		(const WCHAR*)((const char*)system + system->SystemNameOffset),
		system->SystemNameLength / sizeof(WCHAR));
}

//
//    
void CFieldsList::FillObject(const PERF_OBJECT_TYPE* object)
{
	Clear();

	AddHex(TEXT("TotalByteLength"),      object->TotalByteLength);
	AddHex(TEXT("DefinitionLength"),     object->DefinitionLength);
	AddHex(TEXT("HeaderLength"),         object->HeaderLength);
	AddHex(TEXT("ObjectNameTitleIndex"), object->ObjectNameTitleIndex);
	AddTextW(TEXT("ObjectNameTitle"),    object->ObjectNameTitle);
	AddHex(TEXT("ObjectHelpTitleIndex"), object->ObjectHelpTitleIndex);
	AddTextW(TEXT("ObjectHelpTitle"),    object->ObjectHelpTitle);

	AddDetailLevel(TEXT("DetailLevel"),  object->DetailLevel);

	AddHex(TEXT("NumCounters"),          object->NumCounters);
	AddHex(TEXT("DefaultCounter"),       object->DefaultCounter);
	AddHex(TEXT("NumInstances"),         object->NumInstances);
	AddHex(TEXT("CodePage"),             object->CodePage);
	AddLarge(TEXT("PerfTime"),           object->PerfTime);
	AddLarge(TEXT("PerfFreq"),           object->PerfFreq);

	AddText(TEXT(""), TEXT(""));

	AddText(TEXT("ObjectName"), CounterHelpDB::GetName(object->ObjectNameTitleIndex));
	AddLongText(TEXT("ObjectHelp"), CounterHelpDB::GetHelp(object->ObjectHelpTitleIndex));

	if (object->NumInstances == PERF_NO_INSTANCES)
	{
		AddText(TEXT(""), TEXT(""));

		const PERF_COUNTER_BLOCK* pcb = (const PERF_COUNTER_BLOCK*)((const char*)object + object->DefinitionLength);
		AddHex(TEXT("PERF_COUNTER_BLOCK::ByteLength"), pcb->ByteLength);
	}
}

//
//    
void CFieldsList::FillInstance(const PERF_INSTANCE_DEFINITION* instance)
{
	Clear();

	if (instance == NULL)
	{
		//   
		AddText(TEXT("All fields"), TEXT("<FAKE>"));
		return;
	}

	AddHex(TEXT("ByteLength"),             instance->ByteLength);
	AddHex(TEXT("ParentObjectTitleIndex"), instance->ParentObjectTitleIndex);
	AddHex(TEXT("ParentObjectInstance"),   instance->ParentObjectInstance);
	AddHex(TEXT("UniqueID"),               instance->UniqueID);
	AddHex(TEXT("NameOffset"),             instance->NameOffset);
	AddHex(TEXT("NameLength"),             instance->NameLength);

	AddText(TEXT(""), TEXT(""));

	AddText(TEXT("ParentObjectName"), CounterHelpDB::GetName(instance->ParentObjectTitleIndex));

	AddText(TEXT(""), TEXT(""));

	const PERF_COUNTER_BLOCK* pcb = (const PERF_COUNTER_BLOCK*)((const char*)instance + instance->ByteLength);
	AddHex(TEXT("PERF_COUNTER_BLOCK::ByteLength"), pcb->ByteLength);
}

//
//      
void CFieldsList::FillCounter(const PERF_COUNTER_DEFINITION* counter, const PERF_INSTANCE_DEFINITION* instance)
{
	Clear();

	FillCounterFields(counter);

	if (instance == NULL)
	{
		//    ,    
		AddText(TEXT("Value"), TEXT("<FAKE>"));
		return;
	}

	const void* value;
	
	value = (const char*)instance + instance->ByteLength + counter->CounterOffset;

	FillCounterValue(counter, value);
}

//
//      
void CFieldsList::FillCounter(const PERF_COUNTER_DEFINITION* counter, const PERF_OBJECT_TYPE* object)
{
	Clear();

	FillCounterFields(counter);

	if (object == NULL)
	{
		//   .  ..
		AddText(TEXT("Value"), TEXT("<FAKE>"));
		return;
	}

	const void* value;
	
	value = (const char*)object + object->DefinitionLength + counter->CounterOffset;
	
	FillCounterValue(counter, value);
}

//////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////

//
//    ,   
void CFieldsList::FillCounterFields(const PERF_COUNTER_DEFINITION* counter)
{
	AddHex(TEXT("ByteLength"),            counter->ByteLength);
	AddHex(TEXT("CounterNameTitleIndex"), counter->CounterNameTitleIndex);
	AddTextW(TEXT("CounterNameTitle"),    counter->CounterNameTitle);
	AddHex(TEXT("CounterHelpTitleIndex"), counter->CounterHelpTitleIndex);
	AddTextW(TEXT("CounterHelpTitle"),    counter->CounterHelpTitle);
	AddHex(TEXT("DefaultScale"),          counter->DefaultScale);

	AddDetailLevel(TEXT("DetailLevel"), counter->DetailLevel);
	AddCounterType(TEXT("CounterType"), counter->CounterType);

	AddHex(TEXT("CounterSize"),         counter->CounterSize);
	AddHex(TEXT("CounterOffset"),       counter->CounterOffset);

	AddText(TEXT(""), TEXT(""));

	AddText(TEXT("CounterName"),     CounterHelpDB::GetName(counter->CounterNameTitleIndex));
	AddLongText(TEXT("CounterHelp"), CounterHelpDB::GetHelp(counter->CounterHelpTitleIndex));

	AddText(TEXT(""), TEXT(""));
}

//
//     
void CFieldsList::FillCounterValue(const PERF_COUNTER_DEFINITION* counter, const void* value)
{
	CounterTypeMask m;
	CounterTypeMask pure;

	m.dw = counter->CounterType;
	pure.dw = 0;
	pure.type = m.type;

	if (pure.dw == PERF_TYPE_TEXT)
	{
		pure.dw = 0;
		pure.subtype = m.subtype;

		if (pure.dw == PERF_TEXT_ASCII)
		{
			pure.dw = 0;
			pure.size = m.size;

			if (pure.dw == PERF_SIZE_ZERO)
			{
				AddText(TEXT("Value"), (const TCHAR*)value);
			}
			else
			{
				AddText(TEXT("Value"), (const TCHAR*)value, counter->CounterSize);
			}
		}
		else
		{
			pure.dw = 0;
			pure.size = m.size;

			if (pure.dw == PERF_SIZE_ZERO)
			{
				AddTextW(TEXT("Value"), (const WCHAR*)value);
			}
			else
			{
				AddTextW(TEXT("Value"), (const WCHAR*)value, counter->CounterSize/sizeof(WCHAR));
			}
		}
	}
	else if (pure.dw ==  PERF_TYPE_ZERO)
	{
		AddText(TEXT("Value"), TEXT("<ZERO>"));
	}
	else 
	{
		pure.dw = 0;
		pure.size = m.size;

		if (pure.dw == PERF_SIZE_LARGE)
		{
			AddLarge(TEXT("Value"), *(const LARGE_INTEGER*)value);
		}
		else
		{
			AddHex(TEXT("Value"), *(const DWORD*)value);
		}
	}
}

void CFieldsList::AddDetailLevel(const TCHAR* field, int val)
{
	TCHAR* type;
	
	switch (val)
	{
	case PERF_DETAIL_NOVICE:   type = TEXT("PERF_DETAIL_NOVICE"); break;
	case PERF_DETAIL_ADVANCED: type = TEXT("PERF_DETAIL_ADVANCED"); break;
	case PERF_DETAIL_EXPERT:   type = TEXT("PERF_DETAIL_EXPERT"); break;
	case PERF_DETAIL_WIZARD:   type = TEXT("PERF_DETAIL_WIZARD"); break;
	default: type = TEXT("unknown type"); break;
	}

	TCHAR text[64];

	wsprintf(text, TEXT("%s (0x%08x, %d)"), type, val, val);

	AddText(field, text);
}

void CFieldsList::AddCounterType(const TCHAR* field, int val)
{
	CString result;
	CounterTypeMask m;
	CounterTypeMask pure;

	m.dw = val;

	pure.dw = 0;
	pure.size = m.size;

	switch (pure.dw)
	{
	case PERF_SIZE_DWORD:        result += TEXT("PERF_SIZE_DWORD | "); break;
	case PERF_SIZE_LARGE:        result += TEXT("PERF_SIZE_LARGE | "); break;
	case PERF_SIZE_ZERO:         result += TEXT("PERF_SIZE_ZERO | "); break;
	case PERF_SIZE_VARIABLE_LEN: result += TEXT("PERF_SIZE_VARIABLE_LEN | "); break;
	default:                     result += TEXT("unknown_size | ");
	}

	pure.dw = 0;
	pure.type = m.type;

	switch (pure.dw)
	{
	case PERF_TYPE_NUMBER:  result += TEXT("PERF_TYPE_NUMBER | "); break;
	case PERF_TYPE_COUNTER: result += TEXT("PERF_TYPE_COUNTER | "); break;
	case PERF_TYPE_TEXT:    result += TEXT("PERF_TYPE_TEXT | "); break;
	case PERF_TYPE_ZERO:    result += TEXT("PERF_TYPE_ZERO | "); break;
	default:                result += TEXT("unknown_type | ");
	}

	pure.dw = 0;
	pure.type = m.type;

	if (pure.dw == PERF_TYPE_NUMBER)
	{
		pure.dw = 0;
		pure.subtype = m.subtype;

		switch (pure.dw)
		{
		case PERF_NUMBER_HEX:      result += TEXT("PERF_NUMBER_HEX | "); break;
		case PERF_NUMBER_DECIMAL:  result += TEXT("PERF_NUMBER_DECIMAL | "); break;
		case PERF_NUMBER_DEC_1000: result += TEXT("PERF_NUMBER_DEC_1000 | "); break;
		default:                   result += TEXT("unknown_subtype | ");
		}
	}
	else if (pure.dw == PERF_TYPE_COUNTER)
	{
		pure.dw = 0;
		pure.subtype = m.subtype;

		switch (pure.dw)
		{
		case PERF_COUNTER_VALUE:     result += TEXT("PERF_COUNTER_VALUE | "); break;
		case PERF_COUNTER_RATE:      result += TEXT("PERF_COUNTER_RATE | "); break;
		case PERF_COUNTER_FRACTION:  result += TEXT("PERF_COUNTER_FRACTION | "); break;
		case PERF_COUNTER_BASE:      result += TEXT("PERF_COUNTER_BASE | "); break;
		case PERF_COUNTER_ELAPSED:   result += TEXT("PERF_COUNTER_ELAPSED | "); break;
		case PERF_COUNTER_QUEUELEN:  result += TEXT("PERF_COUNTER_QUEUELEN | "); break;
		case PERF_COUNTER_HISTOGRAM: result += TEXT("PERF_COUNTER_HISTOGRAM | "); break;
//		case PERF_COUNTER_PRECISION: result += TEXT("PERF_COUNTER_PRECISION | "); break;
		default:                     result += TEXT("unknown_subtype | ");
		}
	}
	else if (pure.dw == PERF_TYPE_TEXT)
	{
		pure.dw = 0;
		pure.subtype = m.subtype;

		switch (pure.dw)
		{
		case PERF_TEXT_UNICODE: result += TEXT("PERF_TEXT_UNICODE | "); break;
		case PERF_TEXT_ASCII:   result += TEXT("PERF_TEXT_ASCII | "); break;
		default:                result += TEXT("unknown_subtype | ");
		}               
	}

	pure.dw = 0;
	pure.time = m.time;

	switch (pure.dw)
	{
	case PERF_TIMER_TICK:   result += TEXT("PERF_TIMER_TICK | "); break;
	case PERF_TIMER_100NS:  result += TEXT("PERF_TIMER_100NS | "); break;
	case PERF_OBJECT_TIMER: result += TEXT("PERF_OBJECT_TIMER | "); break;
	default:                result += TEXT("unknown_timer | "); break;
	}
	
	pure.dw = 0;
	pure.calc = m.calc;

	if (pure.dw & PERF_DELTA_COUNTER)
	{
		result += TEXT("PERF_DELTA_COUNTER | ");
	}
	
	if (pure.dw & PERF_DELTA_BASE)
	{
		result += TEXT("PERF_DELTA_BASE | ");
	}

	if (pure.dw & PERF_INVERSE_COUNTER)
	{
		result += TEXT("PERF_INVERSE_COUNTER | ");
	}
	
	if (pure.dw & PERF_MULTI_COUNTER)
	{
		result += TEXT("PERF_MULTI_COUNTER | ");
	}

	pure.dw = 0;
	pure.display = m.display;

	switch (pure.dw)
	{
	case PERF_DISPLAY_NO_SUFFIX: result += TEXT("PERF_DISPLAY_NO_SUFFIX"); break;
	case PERF_DISPLAY_PER_SEC:   result += TEXT("PERF_DISPLAY_PER_SEC"); break;
	case PERF_DISPLAY_PERCENT:   result += TEXT("PERF_DISPLAY_PERCENT"); break;
	case PERF_DISPLAY_SECONDS:   result += TEXT("PERF_DISPLAY_SECONDS"); break;
	case PERF_DISPLAY_NOSHOW:    result += TEXT("PERF_DISPLAY_NOSHOW"); break;
	default:                     result += TEXT("unknown_display"); break;
	}

	TCHAR hex[32];
	wsprintf(hex, TEXT(" (0x%08x)"), val);

	result += hex;

	AddLongText(field, result);
}

void CFieldsList::AddHex(const TCHAR* field, int val)
{
	TCHAR text[32];
	wsprintf(text, TEXT("0x%08x (%d)"), val, val);

	AddText(field, text);
}

void CFieldsList::AddTime(const TCHAR* field, const SYSTEMTIME* st)
{
	TCHAR text[64];

	wsprintf(
		text, 
		TEXT("%02d.%02d.%04d %02d:%02d:%02d"), 
		st->wDay, st->wMonth, st->wYear, 
		st->wHour, st->wMinute, st->wSecond);

	AddText(field, text);
}

void CFieldsList::AddLarge(const TCHAR* field, LARGE_INTEGER li)
{
	TCHAR text[64];
	wsprintf(text, TEXT("0x%08x%08x"), li.HighPart, li.LowPart);

	AddText(field, text);
}

void CFieldsList::AddTextW(const TCHAR* field, const WCHAR* val)
{
	AddTextW(field, val, lstrlenW(val));
}

void CFieldsList::AddTextW(const TCHAR* field, const WCHAR* val, int len)
{
#ifdef UNICODE
	
	AddText(field, val, len);

#else //UNICODE

	if (val == NULL)
	{
		AddText(field, NULL);
		return;
	}

	int req = WideCharToMultiByte(CP_ACP, 0, val, len, 0, 0, 0, 0);
	char* valA = (char*)malloc(req + 1);

	WideCharToMultiByte(CP_ACP, 0, val, -1, valA, req + 1, 0, 0);
	valA[req] = 0;

	AddText(field, valA);

	free(valA);

#endif //UNICODE
}

void CFieldsList::AddText(const TCHAR* field, const TCHAR* val, int len)
{
	TCHAR* valT = (TCHAR*)malloc((len + 1) * sizeof(TCHAR));

	memcpy(valT, val, len * sizeof(TCHAR));
	valT[len] = 0;

	AddText(field, valT);

	free(valT);
}

void CFieldsList::AddText(const TCHAR* field, const TCHAR* val)
{
	int index = GetItemCount();

	LVITEM item = {0};

	item.mask    = LVIF_TEXT;
	item.iItem   = index;
	item.pszText = (TCHAR*)field;

	InsertItem(&item);

	if (val == NULL)
	{
		SetItemText(index, 1, TEXT("<NULL>"));
	}
	else
	{
		SetItemText(index, 1, val);
	}
}

void CFieldsList::AddLongText(const TCHAR* field, const TCHAR* val)
{
	int index = GetItemCount();

	LVITEM item = {0};

	item.mask    = LVIF_TEXT;
	item.iItem   = index;
	item.pszText = (TCHAR*)field;

	InsertItem(&item);

	if (val == NULL)
	{
		SetItemText(index, 1, TEXT("<NULL>"));
		return;
	}

	TCHAR text[64];
	int len = lstrlen(val);
	const TCHAR* current = val;

	item.pszText = TEXT("");

	for (;;)
	{
		while (_istspace(*current)) 
		{
			current++;
			len--;
		}

		if (len <= sizeof(text)/sizeof(text[0]) - 1) break;

		int copyed = sizeof(text)/sizeof(text[0]) - 1;

		memcpy(text, current, copyed * sizeof(TCHAR));

		if (!_istspace(current[copyed]))
		{
			while ((copyed > 0) && !_istspace(current[copyed])) copyed--;

			if (copyed == 0) copyed = sizeof(text)/sizeof(text[0]) - 1;
		}

		current += copyed;
		len     -= copyed;

		text[copyed] = 0;

		SetItemText(item.iItem, 1, text);

		item.iItem++;
		InsertItem(&item);
	} 

	SetItemText(item.iItem, 1, current);
}

/////////////////////////////////////////////////////////