//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Viewer
//
//  MODULE:     titledb.cpp
//
//  PURPOSE:          /
//                Matt Pietrek     MSJ March 1996
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//
//  CLASSES:
//      raw_DB
//      title_DB
//

#include "../stdwtl.h"
#include "titledb.h"

//    ,    
// HKEY_PERFORMANCE_DATA.     NT 4.0
#define USE_HKEY_PERFORMANCE_DATA_ONLY

namespace 
{
	//
	// -    
	//  .
	class CHKEY
	{
	public:
		CHKEY() : hkey(0) {}
		~CHKEY() {free();}

		LONG load_HKLM(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_LOCAL_MACHINE;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_LOCAL_MACHINE, &hkey);
		}

		LONG load_HKPD(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_PERFORMANCE_DATA;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_PERFORMANCE_DATA, &hkey);
		}

		HKEY get() const {return hkey;}

		void free()
		{
			if (hkey == 0) return;
			if (hkey == HKEY_LOCAL_MACHINE) return;
			if (hkey == HKEY_PERFORMANCE_DATA) return;
			RegCloseKey(hkey);
		}

	private:

		HKEY hkey;
	};

	const TCHAR* perflibKey = TEXT("Software\\Microsoft\\Windows NT\\CurrentVersion\\Perflib\\");
}

//////////////////////////////////////////////////////////////
//
//   raw_DB
//
//////////////////////////////////////////////////////////////

raw_DB::raw_DB() : _raw_titles(0), _max_title(-1)
{
}

raw_DB::~raw_DB() 
{
	free(_raw_titles);
}

//  
//  FUNCTION:     void Free()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
void raw_DB::Free()
{
	free(_raw_titles);

	_raw_titles = NULL;
	_max_title = -1;
}

//  
//  FUNCTION:     int LoadCounters(const TCHAR*, int)
//
//  PARAMETERS:   [IN] computer     -  
//                [IN] prim_lang_id - Primary language identificator  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:           HKPD  
//
//  COMMENTS:     
// 
int raw_DB::LoadCounters(const TCHAR* computer, int prim_lang_id)
{
	if ((prim_lang_id > 999) || (prim_lang_id < 0))
	{
		_ASSERT(0);
		return ERROR_INVALID_DATA;
	}

	TCHAR title[] = TEXT("Counter 000");

	wsprintf(title, TEXT("Counter %03d"), prim_lang_id);

	return Load(computer, title, TEXT("Last Counter"));
}

//  
//  FUNCTION:     int LoadExplains(const TCHAR*, int)
//
//  PARAMETERS:   [IN] computer     -  
//                [IN] prim_lang_id - Primary language identificator  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:           HKPD  
//
//  COMMENTS:     
// 
int raw_DB::LoadExplains(const TCHAR* computer, int prim_lang_id)
{
	if ((prim_lang_id > 999) || (prim_lang_id < 0))
	{
		_ASSERT(0);
		return ERROR_INVALID_DATA;
	}

	TCHAR title[] = TEXT("Explain 000");

	wsprintf(title, TEXT("Explain %30d"), prim_lang_id);

	return Load(computer, title, TEXT("Last Help"));
}

//
//
//   
//
//

//  
//  FUNCTION:     int Load(const TCHAR* , const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] computer  -  
//                [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKPD  
//
//  COMMENTS:     
// 
int raw_DB::Load(const TCHAR* computer, const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	CHKEY hkpd;
	int res = hkpd.load_HKPD(computer);

	if (res != NO_ERROR)
	{
		return res;
	}

	//  
	res = GetTitles(hkpd.get(), title_str, &_raw_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

#ifndef USE_HKEY_PERFORMANCE_DATA_ONLY

	CHKEY hklm;
	res = hklm.load_HKLM(computer);

	if (res != NO_ERROR)
	{
		Free();
		return res;
	}

	//    .
    res = GetMax(hklm.get(), max_str, &_max_title);

	if (res != NO_ERROR) 
	{	
		Free();
		return res;
	}

#else //USE_HKEY_PERFORMANCE_DATA_ONLY

	GetMax(_raw_titles, &_max_title);

#endif //USE_HKEY_PERFORMANCE_DATA_ONLY
	
	return 0;
}

//  
//  FUNCTION:     int GetMax(HKEY , const TCHAR* , int* )
//
//  PARAMETERS:   [IN]  hklm  - ,    HKLM
//                [IN]  value -    
//                [OUT] max   -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         
//
//  COMMENTS:     
// 
int raw_DB::GetMax(HKEY hklm, const TCHAR* value, int* max)
{
	HKEY hKey = 0;
	LONG res; 

	res = RegOpenKeyEx(hklm, perflibKey, 0, KEY_READ, &hKey);

	if (res != NO_ERROR)
	{
		return res;
	}

	//  , ,    (  ). 
	//       .
 	DWORD cb = sizeof(DWORD);

	res = RegQueryValueEx(hKey, value, 0, 0, (PBYTE)max, &cb);

	RegCloseKey(hKey);

	return res;
}

//  
//  FUNCTION:     int GetMax(const TCHAR*, int* )
//
//  PARAMETERS:   [IN]  hklm  - 
//                [OUT] max   -  
//
//  RETURN VALUE: 0
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         , 
//                   ,   
//                 . ,  .    , 
//                      .
//
//  COMMENTS:     
// 
int raw_DB::GetMax(const TCHAR* titles, int* max)
{
	const TCHAR* temp = titles;
	int max_index = 0;

	for (;;)
	{
		int len = lstrlen(temp);

		if (len == 0) 
		{
			//   
			break;
		}

		int index = _ttoi(temp);
		if (index > max_index) max_index = index;

		//  
		temp += len + 1;

		//     
		temp += lstrlen(temp) + 1;		
	}

	*max = max_index;
	return 0;
}

//  
//  FUNCTION:     int GetTitles(HKEY , const TCHAR* , TCHAR** )
//
//  PARAMETERS:   [IN]  hKey   - ,    
//                [IN]  value  -   
//                [OUT] buffer - 
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          .
//
//  COMMENTS:     
// 
int raw_DB::GetTitles(HKEY hKey, const TCHAR* value, TCHAR** buffer)
{
	LONG res;    
	DWORD size = 0;

	res = RegQueryValueEx(hKey, value, 0, 0, 0, &size);

	if (res != NO_ERROR)
	{
		return res;
	}
			
	TCHAR* tmp = (TCHAR*)malloc(size);

	if (buffer == NULL)
	{
		return ERROR_OUTOFMEMORY;
	}
    
	res = RegQueryValueEx(hKey, value, 0, 0, (PBYTE)tmp, &size);

	if (res != NO_ERROR)
	{
		free(tmp);
		return res;
	}

	*buffer = tmp;
	return NO_ERROR;
}

//////////////////////////////////////////////////////////////
//
//   title_DB
//
//////////////////////////////////////////////////////////////

title_DB::title_DB() : _titles(0), _count_title(0)
{
}

title_DB::~title_DB() 
{
	free(_titles);
}

//  
//  FUNCTION:     void Free()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
void title_DB::Free()
{
	_raw.Free();
	free(_titles);

	_titles = NULL;
	_count_title = 0;
}

//  
//  FUNCTION:     int LoadCounters(const TCHAR*, int)
//
//  PARAMETERS:   [IN] computer     -  
//                [IN] prim_lang_id - Primary language identificator  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          ,  
//
//  COMMENTS:     
// 
int title_DB::LoadCounters(const TCHAR* computer, int prim_lang_id)
{
	Free();

	int res = _raw.LoadCounters(computer, prim_lang_id);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadExplains(const TCHAR*, int)
//
//  PARAMETERS:   [IN] computer     -  
//                [IN] prim_lang_id - Primary language identificator  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          ,  
//
//  COMMENTS:     
// 
int title_DB::LoadExplains(const TCHAR* computer, int prim_lang_id)
{
	Free();

	int res = _raw.LoadExplains(computer, prim_lang_id);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     const TCHAR* Get(int) const
//
//  PARAMETERS:   [IN] n -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          
//
//  COMMENTS:     
// 
const TCHAR* title_DB::Get(int n) const
{
	if (n > _raw.GetLast())
	{
		return TEXT("<index > _max_title>"); 
	}

	if (n < 0)
	{
		return TEXT("<index < 0>"); 
	}

	if (_titles[n] == NULL)
	{
		return TEXT("<_titles[index] == NULL>"); 
	}

	return _titles[n];
}

//  
//  FUNCTION:     int Find(const TCHAR*) const
//
//  PARAMETERS:   [IN] s - 
//
//  RETURN VALUE:   -1
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          ,     -1
//
//  COMMENTS:     
// 
int title_DB::Find(const TCHAR* s) const
{
	int last = _raw.GetLast();

	for (int i = 0; i <= last; i++)
	{
		if (_titles[i] == NULL) continue;
		if (lstrcmpi(s, _titles[i]) == 0) return i;
	}

	return -1;
}

int title_DB::GetLast() const
{
	return _raw.GetLast();
}

int title_DB::GetCount() const
{
	return _count_title;
}

//
//
//   
//
//

//  
//  FUNCTION:     int ParseTitles(const raw_DB&, TCHAR*** )
//
//  PARAMETERS:   [IN]  r    - 
//                [OUT] data -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
int title_DB::ParseTitles(const raw_DB& r, TCHAR*** data)
{
	//     
	TCHAR** tmp = (TCHAR**)malloc((r.GetLast() + 1) * sizeof(TCHAR*));

	if (tmp == NULL)
	{
		return ERROR_OUTOFMEMORY;
	}

	//   
	ZeroMemory(tmp, (r.GetLast() + 1) * sizeof(TCHAR*));

	// ,     const, 
	//     TCHAR*** data
	//   .
	TCHAR* current = (TCHAR*) r.Get();
	int len = 0;

	for (;;_count_title++)
	{
		//   
		len = lstrlen(current);
		
		if (len == 0) 
		{
			//   
			break;
		}

		int index = _ttoi(current);

		//   
		current += len + 1;

		if (index > r.GetLast())
		{
			//   :       .
			//     ,   ..
			//      -  .
			// .. ""      . 
			//Free();
			//return ERROR_INVALID_DATA;
		}
		else
		{
			tmp[index] = current;
		}

		current += lstrlen(current) + 1;
	}

	*data = tmp;
	return NO_ERROR;
}

//////////////////////////////////////////////////////////////