//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Viewer
//
//  MODULE:     titledb_for_sample.cpp
//
//  PURPOSE:          /
//                  -   
//              -       
//                .    
//                    titledb.h  titledb.cpp
//
//                 Matt Pietrek     MSJ March 1996
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//
//  CLASSES:
//      raw_DB
//      title_DB
//

#include "../stdwtl.h"
#include "titledb_for_sample.h"

namespace 
{
	const TCHAR* perflibKey = TEXT("Software\\Microsoft\\Windows NT\\CurrentVersion\\Perflib\\");

	class CHKEY
	{
	public:
		CHKEY() : hkey(0) {}
		~CHKEY() {free();}

		LONG load_HKLM(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_LOCAL_MACHINE;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_LOCAL_MACHINE, &hkey);
		}

		LONG load_HKPD(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_PERFORMANCE_DATA;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_PERFORMANCE_DATA, &hkey);
		}

		LONG load_HKPT(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_PERFORMANCE_TEXT;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_PERFORMANCE_TEXT, &hkey);
		}

		LONG load_HKPNT(const TCHAR* s) 
		{
			if ((s == NULL) || (s[0] == 0))
			{
				hkey = HKEY_PERFORMANCE_NLSTEXT;
				return NO_ERROR;
			}
			return RegConnectRegistry(s, HKEY_PERFORMANCE_NLSTEXT, &hkey);
		}

		HKEY get() const {return hkey;}

		void free()
		{
			if (hkey == 0) return;
			if (hkey == HKEY_LOCAL_MACHINE) return;
			if (hkey == HKEY_PERFORMANCE_DATA) return;
			if (hkey == HKEY_PERFORMANCE_TEXT) return;
			if (hkey == HKEY_PERFORMANCE_NLSTEXT) return;
			RegCloseKey(hkey);
		}

	private:

		HKEY hkey;
	};
}

//////////////////////////////////////////////////////////////
//
//   raw_DB
//
//////////////////////////////////////////////////////////////

raw_DB::raw_DB() 
			: _raw_titles(0), _max_title(-1), _lang_id(TEXT("009"))
{
}

raw_DB::~raw_DB() 
{
	free(_raw_titles);
}

//  
//  FUNCTION:     int LoadHKLM(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKLM  
//
//  COMMENTS:     
// 
int raw_DB::LoadHKLM(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	CHKEY hklm;

	int res = hklm.load_HKLM(_computer);

	if (res != NO_ERROR)
	{
		return res;
	}

	//    .
    res = GetMax(hklm.get(), max_str, &_max_title);

	if (res != NO_ERROR) 
	{
		return res;
	}
	
	HKEY hKey;
	CString key = perflibKey;
	key += _lang_id;

	//  ,  
	res = RegOpenKeyEx(hklm.get(), key, 0, KEY_READ, &hKey);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	//  
	res = GetTitles(hKey, title_str, &_raw_titles);

	RegCloseKey(hKey);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPD(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKPD  
//
//  COMMENTS:     
// 
int raw_DB::LoadHKPD(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	CHKEY hklm;
	int res = hklm.load_HKLM(_computer);

	if (res != NO_ERROR)
	{
		return res;
	}

	//    .
    res = GetMax(hklm.get(), max_str, &_max_title);

	if (res != NO_ERROR) 
	{	
		return res;
	}
	
	CHKEY hkpd;
	res = hkpd.load_HKPD(_computer);

	if (res != NO_ERROR)
	{
		Free();
		return res;
	}

	CString value = title_str;
	value += _lang_id;

	//  
	res = GetTitles(hkpd.get(), value, &_raw_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPT(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKEY_PERFORMANCE_TEXT  
//
//  COMMENTS:     
// 
int raw_DB::LoadHKPT(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	CHKEY hklm;

	int res = hklm.load_HKLM(_computer);

	if (res != NO_ERROR)
	{
		return res;
	}

	//    .
    res = GetMax(hklm.get(), max_str, &_max_title);

	if (res != NO_ERROR) 
	{
		return res;
	}
	
	CHKEY hkpt;

	res = hkpt.load_HKPT(_computer);

	if (res != NO_ERROR)
	{
		Free();
		return res;
	}

	//  
	res = GetTitles(hkpt.get(), title_str, &_raw_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPTN(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKEY_PERFORMANCE_NLSTEXT  
//
//  COMMENTS:     
// 
int raw_DB::LoadHKPNT(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	CHKEY hklm;

	int res = hklm.load_HKLM(_computer);

	if (res != NO_ERROR)
	{
		return res;
	}

	//    .
    res = GetMax(hklm.get(), max_str, &_max_title);

	if (res != NO_ERROR) 
	{
		return res;
	}
	
	CHKEY hkpnt;

	res = hkpnt.load_HKPNT(_computer);

	if (res != NO_ERROR)
	{
		Free();
		return res;
	}

	//  
	res = GetTitles(hkpnt.get(), title_str, &_raw_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     void SetRemote(const TCHAR*)
//
//  PARAMETERS:   [IN] s -       NULL
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       ,     
//                  NULL   ,  
//                 .
//
//  COMMENTS:     
// 
void raw_DB::SetRemote(const TCHAR* s)
{
	if ((s == NULL) || (s[0] == 0))
	{
		_computer.Empty();
	}
	else
	{
		_computer = s;
	}
}

//  
//  FUNCTION:     void SetLangId(const TCHAR*)
//
//  PARAMETERS:   [IN] s -  LangId
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       LangId  .
//
//  COMMENTS:     
// 
void raw_DB::SetLangId(const TCHAR* s)
{
	_lang_id = s;
}

//  
//  FUNCTION:     void Free()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
void raw_DB::Free()
{
	free(_raw_titles);

	_raw_titles = NULL;
	_max_title = -1;
}

//
//
//   
//
//

//  
//  FUNCTION:     int GetMax(HKEY , const TCHAR* , int* )
//
//  PARAMETERS:   [IN]  hklm  - ,    HKLM
//                [IN]  value -    
//                [OUT] max   -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:         
//
//  COMMENTS:     
// 
int raw_DB::GetMax(HKEY hklm, const TCHAR* value, int* max)
{
	HKEY hKey = 0;
	LONG res; 

	res = RegOpenKeyEx(hklm, perflibKey, 0, KEY_READ, &hKey);

	if (res != NO_ERROR)
	{
		return res;
	}

	//  , ,    ( ). 
	//        .
 	DWORD cb = sizeof(DWORD);

	res = RegQueryValueEx(hKey, value, 0, 0, (PBYTE)max, &cb);

	RegCloseKey(hKey);

	return res;
}

//  
//  FUNCTION:     int GetTitles(HKEY , const TCHAR* , TCHAR** )
//
//  PARAMETERS:   [IN]  hKey   - ,    
//                [IN]  value  -   
//                [OUT] buffer - 
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          .
//
//  COMMENTS:     
// 
int raw_DB::GetTitles(HKEY hKey, const TCHAR* value, TCHAR** buffer)
{
	LONG res;    
	DWORD size = 0;

	res = RegQueryValueEx(hKey, value, 0, 0, 0, &size);

	if (res != NO_ERROR)
	{
		return res;
	}
			
	TCHAR* tmp = (TCHAR*)malloc(size);

	if (buffer == NULL)
	{
		return ERROR_OUTOFMEMORY;
	}
    
	res = RegQueryValueEx(hKey, value, 0, 0, (PBYTE)tmp, &size);

	if (res != NO_ERROR)
	{
		free(tmp);
		return res;
	}

	*buffer = tmp;
	return NO_ERROR;
}

//////////////////////////////////////////////////////////////
//
//   title_DB
//
//////////////////////////////////////////////////////////////

title_DB::title_DB() 
			: _titles(0), _count_title(0)
{
}

title_DB::~title_DB() 
{
	free(_titles);
}

//  
//  FUNCTION:     int LoadHKLM(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKLM  ,  
//
//  COMMENTS:     
// 
int title_DB::LoadHKLM(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	int res = _raw.LoadHKLM(title_str, max_str);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPD(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKPD  ,  
//
//  COMMENTS:     
// 
int title_DB::LoadHKPD(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	int res = _raw.LoadHKPD(title_str, max_str);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPT(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKEY_PERFORMANCE_TEXT  
//                ,  
//
//  COMMENTS:     
// 
int title_DB::LoadHKPT(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	int res = _raw.LoadHKPT(title_str, max_str);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     int LoadHKPNT(const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] title_str - ,    
//                [IN] max_str   - ,     
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          HKEY_PERFORMANCE_NLSTEXT  
//                ,  
//
//  COMMENTS:     
// 
int title_DB::LoadHKPNT(const TCHAR* title_str, const TCHAR* max_str)
{
	Free();

	int res = _raw.LoadHKPNT(title_str, max_str);

	if (res != NO_ERROR) 
	{
		return res;
	}

	//  
	res = ParseTitles(_raw, &_titles);

	if (res != NO_ERROR) 
	{
		Free();
		return res;
	}

	return 0;
}

//  
//  FUNCTION:     void Free()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
void title_DB::Free()
{
	_raw.Free();
	free(_titles);

	_titles = NULL;
	_count_title = 0;
}

//  
//  FUNCTION:     const TCHAR* Get(int) const
//
//  PARAMETERS:   [IN] n -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          
//
//  COMMENTS:     
// 
const TCHAR* title_DB::Get(int n) const
{
	if (n > _raw.GetLast())
	{
		return TEXT("<index > _max_title>"); 
	}

	if (n < 0)
	{
		return TEXT("<index < 0>"); 
	}

	if (_titles[n] == NULL)
	{
		return TEXT("<_titles[index] == NULL>"); 
	}

	return _titles[n];
}

//  
//  FUNCTION:     int Find(const TCHAR*) const
//
//  PARAMETERS:   [IN] s - 
//
//  RETURN VALUE:   0
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:          ,     0
//
//  COMMENTS:     
// 
int title_DB::Find(const TCHAR* s) const
{
	int last = _raw.GetLast();

	for (int i = 0; i <= last; i++)
	{
		if (_titles[i] == NULL) continue;
		if (lstrcmpi(s, _titles[i]) == 0) return i;
	}

	return 0;
}

int title_DB::GetLast() const
{
	return _raw.GetLast();
}

int title_DB::GetCount() const
{
	return _count_title;
}

void title_DB::SetRemote(const TCHAR* s)
{
	_raw.SetRemote(s);
}

void title_DB::SetLangId(const TCHAR* s)
{
	_raw.SetLangId(s);
}

//
//
//   
//
//

//  
//  FUNCTION:     int ParseTitles(const raw_DB&, TCHAR*** )
//
//  PARAMETERS:   [IN]  r    - 
//                [OUT] data -  
//
//  RETURN VALUE:   
//
//  REVISION:     20.04.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
int title_DB::ParseTitles(const raw_DB& r, TCHAR*** data)
{
	//     
	TCHAR** tmp = (TCHAR**)malloc((r.GetLast() + 1) * sizeof(TCHAR*));

	if (tmp == NULL)
	{
		return ERROR_OUTOFMEMORY;
	}

	//   
	ZeroMemory(tmp, (r.GetLast() + 1) * sizeof(TCHAR*));

	// ,     const, 
	//     TCHAR*** data
	//   .
	TCHAR* current = (TCHAR*) r.Get();
	int len = 0;

	for (;;_count_title++)
	{
		//   
		len = lstrlen(current);
		
		if (len == 0) 
		{
			//   
			break;
		}

		int index = _ttoi(current);

		//   
		current += len + 1;

		if (index > r.GetLast())
		{
			//   :       .
			//    ,   ..
			//      -  .
			//Free();
			//return ERROR_INVALID_DATA;
		}
		else
		{
			tmp[index] = current;
		}

		current += lstrlen(current) + 1;
	}

	*data = tmp;
	return NO_ERROR;
}

//////////////////////////////////////////////////////////////